package gnu.crypto.jce;

// ----------------------------------------------------------------------------
// $Id: DSSKeyPairGeneratorSpi.java,v 1.1 2002/01/17 11:49:40 raif Exp $
//
// Copyright (C) 2001, 2002 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License or (at your option) any
// later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; see the file COPYING.  If not, write to the
//
//    Free Software Foundation Inc.,
//    59 Temple Place - Suite 330,
//    Boston, MA 02111-1307
//    USA
//
// As a special exception, if you link this library with other files to produce
// an executable, this library does not by itself cause the resulting
// executable to be covered by the GNU General Public License.  This exception
// does not however invalidate any other reasons why the executable file might
// be covered by the GNU General Public License.
// ----------------------------------------------------------------------------

import gnu.crypto.Registry;
import gnu.crypto.sig.dss.DSSKeyPairGenerator;

import java.security.InvalidAlgorithmParameterException;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.DSAParameterSpec;
import java.util.HashMap;

/**
 * The implementation of a {@link java.security.KeyPairGenerator} adapter class
 * to wrap gnu.crypto DSS keypair generator instances.<p>
 *
 * In case the client does not explicitly initialize the KeyPairGenerator (via
 * a call to an <code>initialize()</code> method), the GNU Crypto provider
 * uses a default <i>modulus</i> size (keysize) of 1024 bits.<p>
 *
 * @version $Revision: 1.1 $
 */
public class DSSKeyPairGeneratorSpi extends KeyPairGeneratorAdapter {

   // Constants and variables
   // -------------------------------------------------------------------------

   // Constructor(s)
   // -------------------------------------------------------------------------

   public DSSKeyPairGeneratorSpi() {
      super(Registry.DSS_KPG);
   }

   // Class methods
   // -------------------------------------------------------------------------

   // Instance methods
   // -------------------------------------------------------------------------

   public void initialize(int keysize, SecureRandom random) {
      HashMap attributes = new HashMap();
      attributes.put(DSSKeyPairGenerator.MODULUS_LENGTH, new Integer(keysize));
      if (random != null) {
         attributes.put(DSSKeyPairGenerator.SOURCE_OF_RANDOMNESS, random);
      }

      adaptee.setup(attributes);
   }

   public void initialize(AlgorithmParameterSpec params, SecureRandom random)
   throws InvalidAlgorithmParameterException {
      HashMap attributes = new HashMap();
      if (params != null) {
         if (!(params instanceof DSAParameterSpec)) {
            throw new InvalidAlgorithmParameterException("params");
         }

         attributes.put(DSSKeyPairGenerator.DSS_PARAMETERS, params);
      }

      if (random != null) {
         attributes.put(DSSKeyPairGenerator.SOURCE_OF_RANDOMNESS, random);
      }

      adaptee.setup(attributes);
   }
}
